"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var Utils_1 = require("./Utils");
var EventObject_1 = require("./EventObject");
var SearchPickerResults = /** @class */ (function (_super) {
    __extends(SearchPickerResults, _super);
    function SearchPickerResults(sourceElm, options) {
        var _this = _super.call(this) || this;
        _this.sourceElm = sourceElm;
        _this.options = options;
        _this.hideOnEmptyResults = false;
        _this.selectFirstOnDefault = true;
        _this.isShown = false;
        _this.trackSelectedItems = true;
        _this.items = [];
        _this.itemsToHide = [];
        _this.selectedObj = {};
        _this.lastSearchQuery = null;
        _this.processSearchResponses = true;
        _this.setupHtml();
        _this.bindEvents();
        _this.highlightedIndex = _this.getDefaultHighlightedIndex();
        return _this;
    }
    SearchPickerResults.prototype.hide = function () {
        if (!this.isShown)
            return;
        this.dropdownElm.style.display = 'none';
        this.isShown = false;
    };
    SearchPickerResults.prototype.show = function () {
        if (this.isShown)
            return;
        this.dropdownElm.style.display = 'block';
        this.isShown = true;
        this.invalidateActiveResultByIndex();
    };
    SearchPickerResults.prototype.search = function (query) {
        if (this.lastSearchQuery === query) {
            if (this.hideOnEmptyResults && this.items.length === 0)
                this.hide();
            else
                this.show();
            return;
        }
        this.searchInternal(query);
    };
    SearchPickerResults.prototype.repositionResults = function () {
        this.setTop(this.sourceElm.offsetHeight);
    };
    SearchPickerResults.prototype.moveSelectedDown = function () {
        if (!this.isShown)
            return;
        this.highlightedIndex++;
        if (this.highlightedIndex > this.items.length - 1) {
            this.highlightedIndex = this.items.length - 1;
        }
        this.invalidateActiveResultByIndex();
    };
    SearchPickerResults.prototype.moveSelectedUp = function () {
        if (!this.isShown)
            return;
        this.highlightedIndex--;
        if (this.highlightedIndex <= this.getDefaultHighlightedIndex()) {
            this.highlightedIndex = this.getDefaultHighlightedIndex();
        }
        this.invalidateActiveResultByIndex();
    };
    SearchPickerResults.prototype.selectHighlighted = function (selectedVia) {
        if (!this.highlightedEl)
            return;
        var id = Utils_1.Utility.getAttribute(this.highlightedEl, 'data-id');
        var item = this.getItemById(id);
        if (item != null)
            this.$notifyEvent('resultSelected', item, this.lastSearchQuery, selectedVia, this.items.indexOf(item) + 1, this.items.length);
        this.setProcessSearchResponses(false);
    };
    SearchPickerResults.prototype.setTop = function (top) {
        if (top <= 0)
            top = 0;
        this.dropdownElm.style.top = top + 'px';
    };
    SearchPickerResults.prototype.addSelectedResult = function (result) {
        if (!this.trackSelectedItems)
            return; //option disabled
        if (!this.selectedObj[result.id]) {
            this.selectedObj[result.id] = true;
            this.invalidateSelectedResults();
            this.clearHighlighted();
        }
    };
    SearchPickerResults.prototype.removeSelectedResult = function (item) {
        if (!this.trackSelectedItems)
            return; //option disabled
        if (!this.isSelected(item.id.toString()))
            return;
        delete this.selectedObj[item.id.toString()];
        this.invalidateSelectedResults();
    };
    SearchPickerResults.prototype.updateOptions = function (options) {
        this.options = options;
    };
    SearchPickerResults.prototype.addToBlacklist = function (items) {
        if (!items) {
            return;
        }
        var toAdd;
        if (!Array.isArray(items)) {
            toAdd = [items];
        }
        else {
            toAdd = items;
        }
        this.itemsToHide = this.itemsToHide.concat(toAdd);
        if (toAdd.length) {
            this.invalidateSelectedResults();
        }
    };
    SearchPickerResults.prototype.removeFromBlacklist = function (items) {
        if (!items) {
            return;
        }
        var toRemove;
        if (!Array.isArray(items)) {
            toRemove = [items];
        }
        else {
            toRemove = items;
        }
        if (toRemove.length) {
            for (var _i = 0, toRemove_1 = toRemove; _i < toRemove_1.length; _i++) {
                var i = toRemove_1[_i];
                for (var b = this.itemsToHide.length - 1; b >= 0; b--) {
                    if (i.id === this.itemsToHide[b].id) {
                        this.itemsToHide.splice(b, 1);
                    }
                }
            }
            this.invalidateSelectedResults();
        }
    };
    SearchPickerResults.prototype.getHighlighted = function () {
        if (this.highlightedEl) {
            var hi = this.getItemById(this.highlightedEl.getAttribute("data-id"));
            if (hi) {
                return hi;
            }
        }
    };
    /* устанавливаем флаг, что не нужно обрабатывать результаты поиска
        (напр.  из-зп того что поле сейчас пустое) */
    SearchPickerResults.prototype.setProcessSearchResponses = function (process) {
        this.processSearchResponses = process;
    };
    SearchPickerResults.prototype.isSelected = function (id) {
        return typeof (this.selectedObj[id]) !== 'undefined';
    };
    SearchPickerResults.prototype.searchInternal = function (query) {
        var _this = this;
        this.lastSearchQuery = query;
        this.setProcessSearchResponses(true);
        return this.options.searcher.search(query, this.options, function (items, data) {
            if (query !== _this.lastSearchQuery || !_this.processSearchResponses) {
                return;
            }
            _this.resultsElm.innerHTML = '';
            _this.items = items;
            var numResults = 0;
            if (items.length) {
                for (var i = 0; i < items.length; i++) {
                    //блеклист
                    if (_this.itemsToHide.indexOf(items[i]) > -1 || _this.isSelected(items[i].id)) {
                        continue;
                    }
                    //лимит на показываемые результаты
                    if (_this.options.resultsLimit && i >= _this.options.resultsLimit) {
                        break;
                    }
                    var result = _this.$buildResult(items[i], query);
                    if (result != null) {
                        _this.resultsElm.appendChild(result);
                        numResults++;
                    }
                }
            }
            _this.clearHighlighted();
            if (numResults == 0) {
                if (_this.hideOnEmptyResults) {
                    _this.hide();
                    return;
                }
                else {
                    _this.resultsElm.appendChild(_this.$buildNoResults(query));
                }
            }
            if (_this.options.resultFooterRenderer) {
                var footer = _this.$buildResultsFooter(query, items, data);
                if (footer != null) {
                    _this.resultsElm.appendChild(footer);
                }
            }
            _this.$notifyEvent('afterSearch', query, items, numResults);
            _this.show();
        }, function (message) {
            _this.resultsElm.innerHTML = '';
            _this.resultsElm.appendChild(_this.$buildErrorResult(message));
        });
    };
    SearchPickerResults.prototype.setupHtml = function () {
        this.dropdownElm = document.createElement('div');
        this.dropdownElm.className = 'dropdown';
        Utils_1.Utility.insertAfter(this.dropdownElm, this.sourceElm);
        this.resultsElm = this.$template();
        this.dropdownElm.style.left = '0';
        this.dropdownElm.style.display = 'none';
        this.dropdownElm.appendChild(this.resultsElm);
    };
    SearchPickerResults.prototype.bindEvents = function () {
        var _this = this;
        this.dropdownElm.onclick = function (e) {
            e.preventDefault();
            e.stopPropagation();
            var result = e.target;
            while (result != null) {
                if (Utils_1.Utility.hasClass(result, 'result'))
                    break;
                result = result['parentElement'];
            }
            if (!result || Utils_1.Utility.hasClass(result, 'disabled'))
                return false;
            var id = Utils_1.Utility.getAttribute(result, 'data-id');
            var item = _this.getItemById(id);
            if (item != null)
                _this.$notifyEvent('resultSelected', item, _this.lastSearchQuery, 'click', _this.items.indexOf(item) + 1, _this.items.length);
            return false;
        };
    };
    SearchPickerResults.prototype.$buildResult = function (item, query, data) {
        var result = document.createElement('li');
        result.className = 'result';
        var idStr = item.id.toString();
        result.setAttribute('data-id', idStr);
        if (this.isSelected(idStr))
            result.className += ' disabled';
        result.appendChild(this.options.resultRenderer(item, query));
        return result;
    };
    SearchPickerResults.prototype.$buildResultsFooter = function (query, results, data) {
        var footer = document.createElement('div');
        footer.className = 'result-footer';
        if (this.options.resultFooterRenderer) {
            var footerContent = this.options.resultFooterRenderer(query, results, data);
            if (footerContent) {
                footer.appendChild(footerContent);
            }
            else {
                return null;
            }
        }
        return footer;
    };
    SearchPickerResults.prototype.clearHighlighted = function () {
        this.highlightedIndex = this.getDefaultHighlightedIndex();
        //this.highlightedEl = null;
        this.invalidateActiveResultByIndex();
    };
    SearchPickerResults.prototype.$buildNoResults = function (query) {
        var noResult = document.createElement('li');
        noResult.className = 'no-results';
        noResult.appendChild(this.options.noResultsRenderer(query));
        return noResult;
    };
    SearchPickerResults.prototype.$buildErrorResult = function (message) {
        var errorResult = document.createElement('li');
        errorResult.className = 'no-results';
        errorResult.appendChild(document.createTextNode(message));
        return errorResult;
    };
    SearchPickerResults.prototype.$template = function () {
        var ul = document.createElement('ul');
        ul.className = 'dropdown-menu';
        return ul;
    };
    SearchPickerResults.prototype.getItemById = function (id) {
        for (var i = 0; i < this.items.length; i++) {
            if (this.items[i].id.toString() === id)
                return this.items[i];
        }
        return null;
    };
    SearchPickerResults.prototype.invalidateSelectedResults = function () {
        var _this = this;
        this.$foreachResult(function (result) {
            var id = Utils_1.Utility.getAttribute(result, 'data-id');
            if (_this.isSelected(id) || _this.itemsToHide.some(function (item) { return item.id === id; })) {
                Utils_1.Utility.removeClass(result, 'highlighted');
                Utils_1.Utility.addClass(result, 'disabled');
            }
            else {
                Utils_1.Utility.removeClass(result, 'disabled');
            }
            return true;
        });
    };
    SearchPickerResults.prototype.invalidateActiveResultByIndex = function () {
        var _this = this;
        var hoverRemoved = false, elemFound = false;
        var index = 0;
        var beforeId = this.highlightedEl ? this.highlightedEl.getAttribute("data-id") : -1;
        this.highlightedEl = null;
        this.$foreachResult(function (result) {
            if (Utils_1.Utility.hasClass(result, 'disabled') || Utils_1.Utility.hasClass(result, 'no-results'))
                return true; //continue
            if (!elemFound && index === _this.highlightedIndex) {
                _this.highlightedEl = result;
                elemFound = true;
            }
            else if (!hoverRemoved && Utils_1.Utility.hasClass(result, 'highlighted')) {
                Utils_1.Utility.removeClass(result, 'highlighted');
                hoverRemoved = true;
            }
            if (elemFound && hoverRemoved)
                return false; //break
            index++;
            return true; //continue
        });
        var afterId = this.highlightedEl ? this.highlightedEl.getAttribute("data-id") : -1;
        this.$notifyEvent("highlight", {
            item: afterId ? this.getItemById(afterId) : undefined,
            hasChanged: beforeId !== afterId
        });
        this.repositionResults();
        if (this.highlightedEl) {
            Utils_1.Utility.addClass(this.highlightedEl, 'highlighted');
            this.invalidateActiveResultVisible(this.highlightedEl);
        }
    };
    SearchPickerResults.prototype.invalidateActiveResult = function () {
        var _this = this;
        if (!this.highlightedEl)
            return;
        var hoverRemoved = false, itemFound = false;
        var index = 0;
        this.$foreachResult(function (result) {
            if (Utils_1.Utility.hasClass(result, 'disabled'))
                return true;
            if (!itemFound && _this.highlightedEl === result) {
                _this.highlightedIndex = index;
                itemFound = true;
            }
            else if (!hoverRemoved && Utils_1.Utility.hasClass(result, 'highlighted')) {
                Utils_1.Utility.removeClass(result, 'highlighted');
                hoverRemoved = true;
            }
            if (itemFound && hoverRemoved)
                return false; //break;
            index++;
            return true; //continue
        });
        Utils_1.Utility.addClass(this.highlightedEl, 'highlighted');
    };
    SearchPickerResults.prototype.$foreachResult = function (cb) {
        var liCollection = this.resultsElm.childNodes;
        for (var i = 0; i < liCollection.length; i++) {
            if (!cb(liCollection[i]))
                break;
        }
    };
    SearchPickerResults.prototype.invalidateActiveResultVisible = function (el) {
        var resultsJsObj = this.resultsElm;
        var maxHeight = parseInt(Utils_1.Utility.getCssPropertyValue(resultsJsObj, 'maxHeight') || "190px", 10);
        var scrollTop = resultsJsObj.scrollTop;
        var visibleBottom = maxHeight + scrollTop;
        var elTop = el.offsetTop;
        var elBottom = elTop + el.offsetHeight;
        if (elBottom >= visibleBottom) {
            resultsJsObj.scrollTop = scrollTop + (elBottom - visibleBottom);
        }
        else if (el.offsetTop < scrollTop) {
            resultsJsObj.scrollTop = el.offsetTop;
        }
    };
    SearchPickerResults.prototype.getDefaultHighlightedIndex = function () {
        return this.selectFirstOnDefault ? 0 : -1;
    };
    return SearchPickerResults;
}(EventObject_1.EventObject));
exports.SearchPickerResults = SearchPickerResults;
//# sourceMappingURL=SearchPickerResults.js.map