"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var Utility = /** @class */ (function () {
    function Utility() {
    }
    Utility.wrapResultText = function (source, query) {
        var nodes = [];
        var startpos;
        var zregex = new RegExp(query.replace(/[-[\]{}()*+?.,\\^$|#\s]/g, "\\$&"), 'i');
        if (query.length) {
            startpos = source.search(zregex);
            if (startpos < 0) {
                nodes.push(document.createTextNode(source));
            }
            else {
                nodes.push(document.createTextNode(source.substr(0, startpos)));
                var em = document.createElement('em');
                em.appendChild(document.createTextNode(source.substr(0, startpos + query.length).substr(startpos)));
                nodes.push(em);
                nodes.push(document.createTextNode(source.substr(startpos + query.length)));
            }
        }
        else {
            nodes.push(document.createTextNode(source));
        }
        return nodes;
    };
    Utility.filterItems = function (items, query, options) {
        query = Utility.ltrim(query); //trim input query
        var regex = new RegExp('^' + query.replace(/[-[\]{}()*+?.,\\^$|#\s]/g, "\\$&"), 'i');
        var ref = items.slice(0);
        var foundItems = [];
        for (var i = 0; i < ref.length; i++) {
            var item = ref[i];
            var itemName = item.title;
            if (options.searchInValues) {
                itemName += ' ' + item.id.replace(/^\w+\\/g, '');
            }
            if (regex.test(item.title)) {
                foundItems.push(item);
            }
            else if (itemName.indexOf(' ') >= 0 || itemName.indexOf('[') === 0 || itemName.indexOf('\\') >= 0) {
                var parts = itemName.replace(/[\[]/g, '').replace(/\\/g, ' ').split(' ');
                if (parts.length) {
                    for (var j = 0; j < parts.length; j++) {
                        var part = parts[j];
                        if (regex.test(part)) {
                            foundItems.push(item);
                            break;
                        }
                    }
                }
            }
        }
        return foundItems;
    };
    Utility.parseHtml = function (html) {
        var root = document.createElement("div");
        root.innerHTML = html;
        return root.firstChild;
    };
    Utility.hasClass = function (elem, clsName) {
        if (elem.classList) {
            return elem.classList.contains(clsName);
        }
        //IE fallback:
        return elem.className.match(new RegExp('(\\s|^)' + clsName + '(\\s|$)')) != null;
    };
    Utility.getAttribute = function (element, attrName) {
        var attr = element.attributes[attrName];
        if (!attr)
            return undefined;
        return attr.value;
    };
    Utility.htmlEncode = function (value) {
        var div = document.createElement('div');
        var text = document.createTextNode(value);
        div.appendChild(text);
        return div.innerHTML;
    };
    Utility.addClass = function (elem, clsName) {
        if (elem.classList) {
            elem.classList.add(clsName);
            return;
        }
        if (!Utility.hasClass(elem, clsName))
            elem.className += " " + clsName;
    };
    Utility.removeClass = function (elem, clsName) {
        if (Utility.hasClass(elem, clsName)) {
            if (elem.classList) {
                elem.classList.remove(clsName);
                return;
            }
            var reg = new RegExp('(\\s|^)' + clsName + '(\\s|$)');
            elem.className = elem.className.replace(reg, ' ');
        }
    };
    Utility.getCssPropertyValue = function (elem, propname) {
        var computedStyle;
        if (window.getComputedStyle) {
            computedStyle = window.getComputedStyle(elem, null);
        }
        else {
            computedStyle = elem.currentStyle;
        }
        return computedStyle.getPropertyValue(propname);
    };
    Utility.isArray = function (obj) {
        return Array.isArray(obj);
    };
    Utility.isString = function (obj) {
        return typeof (obj) === 'string';
    };
    Utility.trim = function (source) {
        if (!String.prototype.trim) {
            return source.replace(/^\s+|\s+$/g, '');
        }
        return source.trim();
    };
    Utility.ltrim = function (source) {
        return source.replace(/^\s+/, '');
    };
    Utility.insertAfter = function (elem, refElem) {
        var parent = refElem.parentNode;
        var next = refElem.nextSibling;
        if (next) {
            return parent.insertBefore(elem, next);
        }
        else {
            return parent.appendChild(elem);
        }
    };
    return Utility;
}());
exports.Utility = Utility;
//# sourceMappingURL=Utils.js.map